part of mvu_flutter;

/// [Program]'s view widget.
///
/// Base UI widget that can be extended instead of [StatelessWidget]. Must be a
/// descendant of [Program] with the same [Model] type or will throw an exception.
/// Everything is the same as with [StatelessWidget], but [build] method is replaced
/// by [view] method and it now take additional argument – a [model].
///
/// Note - be careful when specifying the generic. Using [View<A>] as a descendant
/// of [Program] tree with the structure as [Program<A>] -> [Program<B>] can lead
/// to bugs. Every [View] must strictly use its direct [Program]'s [Model] type.
abstract class View<ModelType extends Model, MsgType extends Msg>
    extends StatelessWidget {
  View() {
    _Runtime.isProgramExists<ModelType, MsgType>();
  }

  /// Default update function that takes message and updates model
  /// accordingly
  Future<void> dispatch(MsgType message) =>
      dispatchGlobal<ModelType, MsgType>(message);

  /// Does the same as default update function, but returns a closure
  /// that is easy to pass as a callback, e.g. as an argument in onTap
  void Function() dispatchCallback(MsgType message) => () => dispatch(message);

  /// Main UI method that must be overriden instead of build.
  ///
  /// Takes as an argument BuildContext and current model of View's type
  Widget view(BuildContext context, ModelType model);

  @override
  Widget build(BuildContext context) =>
      view(context, _Runtime.getManager<ModelType, MsgType>()!.model.value);
}
